# -*- coding: UTF-8 -*-
import os
import sys
import MySQLdb
import logging.handlers
import subprocess
import config
from datetime import datetime, date, timedelta

log = logging.getLogger('su.artix.optimize_tables')
handler = logging.handlers.RotatingFileHandler('log_optimize_tables.log', maxBytes=1000000, backupCount=10, delay=False)
handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)-8s %(message)s '))
log.addHandler(handler)
log.setLevel(logging.INFO)

def query_yes_no(question, default="yes"):
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)

    while True:
        sys.stdout.write(question + prompt)
        choice = raw_input().lower().strip()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' (or 'y' or 'n').\n")

def optimize(host, port, user, passwd, db, tables):
    try:
        conn = MySQLdb.connect(host=host, port=port, user=user, passwd=passwd, db=db)
        conn.set_character_set('utf8')
        conn.autocommit(True)
    except MySQLdb.Error as e:
        log.error("Error %d: %s" % (e.args[0], e.args[1]))
        sys.exit (1)

    log.info('Начинаем оптимизацию таблиц для БД %s' % db)
    start = datetime.now()
    cursor = conn.cursor()
    
    big_tables = []
    
    df = subprocess.Popen(["df", "-BK", "/var/lib/mysql/%s" % db], stdout=subprocess.PIPE)
    output = df.communicate()[0]
    device, size, used, available, percent, mountpoint = output.split("\n")[1].split()
    available_space = int(filter(str.isdigit, available))*1024
    for t in tables:
        table_size = os.path.getsize('/var/lib/mysql/%s/%s.ibd' % (db, t))
        if available_space <= table_size:
            big_tables.append(t)
    
    optimize_big_tables = False
    
    if len(big_tables) > 0:
        optimize_big_tables = query_yes_no('Размер таблиц %s больше, чем доступно места на жестком диске.\nЕсть вероятность, что не хватит места для оптимизации этих таблиц. Хотите попробовать выполнить оптимизацию для них?' % (big_tables))
    
    for t in tables:
        if len(big_tables) > 0 and t in big_tables and not optimize_big_tables:
            log.warning("Don't start optimize %s because table size grate then available space" % (t))
            continue
        else:
            log.info('Optimize %s' % (t))
            query = 'OPTIMIZE TABLE %s' % (t)
            cursor.execute(query)
    
    conn.close()
    
    log.info('Выполнено за %s' % (datetime.now() - start))

for conf in config.readConfig():
    optimize(conf['host'], conf['port'], conf['user'], conf['passwd'], conf['db'], conf['tables'])
